
import os
import math

import pygame
from pygame.locals import *

from define import *


class Audio:

    # Which files to load on startup. These files can be played by creating an
    # Sfx object with this as filename.
    SOUNDFILES = [
            'boss_activate.ogg',
            'boss_hurt.ogg',
            'boss_proj.ogg',
            'bounce1.ogg',
            'bounce2.ogg',
            'bounce3.ogg',
            'bounce4.ogg',
            'bounce5.ogg',
            'bounce6.ogg',
            'bounce7.ogg',
            'bounce8.ogg',
            'bounce_fire.ogg',
            'bullet_impact.ogg',
            'closed_close.ogg',
            'closed_death.ogg',
            'closed_move1.ogg',
            'closed_move2.ogg',
            'closed_move3.ogg',
            'closed_move4.ogg',
            'closed_open.ogg',
            'dragon_activate.ogg',
            'dragon_death.ogg',
            'dragon_fire.ogg',
            'door_unlock.ogg',
            'ghost_attack.ogg',
            'ghost_death.ogg',
            'ghost_hurt.ogg',
            'ghost_popup.ogg',
            'face_activate.ogg',
            'face_attack.ogg',
            'face_death.ogg',
            'face_move.ogg',
            'flower_charge.ogg',
            'flower_fire.ogg',
            'heart.ogg',
            'key.ogg',
            'key_white.ogg',
            'laser_fire.ogg',
            'nova_explode.ogg',
            'nova_launch.ogg',
            'nova_charge.ogg',
            'rake_hit.ogg',
            'rake_swing.ogg',
            'shadow_attack.ogg',
            'shadow_death.ogg',
            'shadow_move.ogg',
            'skeleton_activate.ogg',
            'skeleton_attack.ogg',
            'skeleton_death.ogg',
            'skeleton_hurt.ogg',
            'spray1.ogg',
            'spray2.ogg',
            'spray3.ogg',
            'spread_fire.ogg',
            'tv_explode.ogg',
            'tv_pickup.ogg',
            'weapon_pickup.ogg'
    ]


    MUSIC_VOL = 0.7
    MUSIC_VOL_LOW = 0.0


    def __init__(self, main):
        self.main = main

        # Init sounds dictionary
        self.sounddict = dict()
        for f in self.SOUNDFILES:
            self.sounddict[f] = pygame.mixer.Sound(os.path.join(DATA_DIR, f))
        self.playing = []
        self.loud = 1
        self.music = None


    def setMusicVol(self, v):
        if v == 1:
            pygame.mixer.music.set_volume(self.MUSIC_VOL)
        else:
            pygame.mixer.music.set_volume(self.MUSIC_VOL_LOW)
        self.loud = v



    def playMusic(self, filename):
        self.loud = 1
        if self.main.core.config.music == 0: return
        self.music = filename
        if filename == None:
            pygame.mixer.music.stop()
        else: 
            filename = os.path.join(DATA_DIR, filename)
            pygame.mixer.music.load(filename)
            pygame.mixer.music.play(0)
            pygame.mixer.music.set_volume(self.MUSIC_VOL)



    def step(self, cam, newsounds):
        # Play any new sounds
        for s in newsounds:
            left, right = self.computePan(cam, s.src, s.mul)
            if s.fn in self.sounddict:
                channel = self.sounddict[s.fn].play()
                if channel == None:
                    # TODO This seems to happen sometimes for some reason.
                    # Generally when several sounds are played simultaneously.
                    pass
                else: 
                    channel.set_volume(left, right)
                    self.playing.append({'sound': s, 'channel': channel})
            else:
                print 'warning: sound ' + s.fn + ' not in sounddict'

        # Update the pan of sounds in the playing list
        for p in self.playing:
            if p['channel'].get_busy():
                left, right = self.computePan(cam, p['sound'].src)
                p['channel'].set_volume(left, right)
            else:
                p['channel'] = None
        # Delete any finished sounds
        self.playing = [p for p in self.playing if p['channel'] != None]
        # Replay music if it is finished.
        if self.music != None and pygame.mixer.music.get_busy() == False:
            pygame.mixer.music.play(0)
            self.setMusicVol(self.loud)


        
    # Compute panning levels of the sound depending on position and direction.
    # No idea whether this is a proper way of doing this, but it seems to work.
    def computePan(self, cam, src, mul=1.0):
        if src == None: return 0.5, 0.5
        dist = sum((src.pos[i] - cam[i])**2 for i in range(3))**0.5
        dist /= mul**2
        dist = max(1.0, dist)
        dir = math.atan2(src.pos[1] - cam[1], src.pos[0] - cam[0])
        dir -= (cam[3]+90.0)/180.0*math.pi
        distmul = 1.0/dist
        leftmul = (math.cos(dir)+1.0)/2.0
        left = leftmul*distmul
        right = (1.0-leftmul)*distmul
        return left, right




