

import math

from OpenGL.GL import *

from define import *
from glarray import *


# Hud class, handles rendering the in-game Hud (showing health, weapons, etc.)
class Hud:


    def __init__(self, main):
        self.main = main


    def draw(self):
        glDisable(GL_DEPTH_TEST)
        glPushMatrix()
        glLoadIdentity()
        # Compute coordinates of screen bottom
        ybot = -float(self.main.core.scr_xx[1])/self.main.core.scr_xx[0]
        vertices = [] 
        texcoords = []

        # Draw everything
        self.drawHeldWeapon(ybot, vertices, texcoords)
        self.drawHealth(-ybot, vertices, texcoords)
        self.drawWeapon(-ybot, vertices, texcoords)
        self.drawAmmo(-ybot, vertices, texcoords)
        self.drawKeys(ybot, vertices, texcoords)
        if self.main.boss: 
            self.drawBossHealth(-ybot, vertices, texcoords)

        n = len(vertices)/3
        vertices = glarray(vertices)
        texcoords = glarray(texcoords)
        glVertexPointer(3,GL_FLOAT,0,vertices)
        glTexCoordPointer(2,GL_FLOAT,0,texcoords)
        glDrawArrays(GL_QUADS,0,n)
        glPopMatrix()
        glEnable(GL_DEPTH_TEST)


    # Draw weapon itself
    def drawHeldWeapon(self, ybot, vertices, texcoords):
        if self.main.pc.weapon_switch < 32:
            ybot -= self.main.pc.weapon_switch/32.0
            self.main.pc.weapons[self.main.weapon_i].draw(
                    ybot, vertices, texcoords)
        else:
            ybot -= (64 - self.main.pc.weapon_switch)/32.0
            self.main.pc.weapons[self.main.pc.weapon_prev].draw(
                    ybot, vertices, texcoords)


    def drawHealth(self, ytop, vertices, texcoords):
        size = 1.0/8
        tex = self.main.tex_hud
        for i in range(self.main.pc.nrg):
            x = -1+i*size; y = ytop
            vertices += [x+size, y, -1.0, x, y, -1.0,
                    x, y-size, -1.0, x+size, y-size, -1.0]
        texcoords += tex.select((0,2),(0.5,0.5))*self.main.pc.nrg


    # Draws the weapon icon, not the weapon itself.
    def drawWeapon(self, ytop, vertices, texcoords):
        size = 1.0/4
        tex = self.main.tex_hud
        x = 1; y = ytop
        vertices += [x, y, -1.0, x-size, y, -1.0,
                x-size, y-size/2, -1.0, x, y-size/2, -1.0]
        weapon = self.main.weapon_i
        texcoords += tex.select((weapon+1,2), (1.0, 0.5))
        


    def drawAmmo(self, ytop, vertices, texcoords):
        if self.main.weapon_i == WEAPON_RAKE: return
        size = 1.0/4
        tex = self.main.tex_hud
        ammo = self.main.ammo[self.main.weapon_i]
        for i in range(10):
            digit = (ammo/10**i)
            if digit == 0: break
            digit %= 10
            x = 1-(2+i)*size/2; y = ytop
            vertices += [x, y, -1.0, x-size/2, y, -1.0,
                    x-size/2, y-size, -1.0, x, y-size, -1.0]
            texcoords += tex.select((digit/2.0,3), (0.5, 1.0))


    def drawKeys(self, ybot, vertices, texcoords):
        x = -1
        size = 1.0/4
        tex = self.main.tex_hud
        keys = list(self.main.mapkeys)
        keys[0] = len([i for i in keys[0] if i == (-1,-1)])
        y = ybot
        for i in range(5):
            for j in range(keys[i]):
                vertices += [x+size/2, y+size, -1.0, x, y+size, -1.0,
                        x, y, -1.0, x+size/2, y, -1.0]
                texcoords += tex.select((5+i/2.0,3), (0.5, 1.0))
                x += size/2


    def drawBossHealth(self, ytop, vertices, texcoords):
        nrg = int(math.ceil(self.main.boss.nrg/10.0))
        size = 1.0/16
        tex = self.main.tex_hud
        for i in range(nrg):
            x = -5.0/16 + size*i/2; y = ytop
            vertices += [x+size/2, y, -1.0, x, y, -1.0,
                    x, y-size, -1.0, x+size/2, y-size, -1.0]
        texcoords += tex.select((8,3),(0.125,0.25))*nrg



