

import math

from define import *


# A Sprite is a game entity shown as billboard. Sprites should be recreated 
# every step when a Thing is drawn.
class Sprite:


    def __init__(self, main, pos, spr):
        self.main = main
        self.main.sprites.append(self)
        self.pos = pos
        self.spr = spr


    # Draw the sprite. 
    # <off> is the corner coordinate offset in the x and y directions, so we 
    # don't need to compute the same sines/cosines a hundred times.
    def draw(self, off, vertices, texcoords):
        # Frustum (well, triangle) culling. The sprite is only drawn when 
        # inside the view frustum.
        cam = self.main.cam
        cd = cam[3]/180.0*math.pi
        sd = math.atan2(self.pos[1]-cam[1], self.pos[0]-cam[0])
        vs = (self.pos[0]-cam[0], self.pos[1]-cam[1])
        hfov = math.pi/4
        v0 = (math.cos(cd-hfov), math.sin(cd-hfov))
        v1 = (math.cos(cd+hfov), math.sin(cd+hfov))
        if (dot(vs, v0) >= -0.5 and dot(vs, v1) > -0.5) or self.main.cheat:
            vertices += [
                    self.pos[0]+off[0], self.pos[1]+off[1], self.pos[2]+0.5,
                    self.pos[0]-off[0], self.pos[1]-off[1], self.pos[2]+0.5,
                    self.pos[0]-off[0], self.pos[1]-off[1], self.pos[2]-0.5,
                    self.pos[0]+off[0], self.pos[1]+off[1], self.pos[2]-0.5]
            tex = self.main.tex_things
            texcoords += tex.select(self.spr)



