

import sys

import pygame
from pygame.locals import *
from OpenGL.GL import *

from glarray import *
from define import *

from StateMain import StateMain


# Menu state, some simple options to start the game with.
class StateMenu:

    def __init__(self, core):
        self.core = core
        self.keys = self.core.keys
        self.tex_hud = self.core.tex_hud
        pygame.mouse.set_visible(True)
        pygame.event.set_grab(False)
        glDisableClientState(GL_COLOR_ARRAY)


    def step(self):
        if (self.keys[KEY_1] == 3):
            save = self.startSave()
            self.core.states.append(StateMain(self.core, save))
        if (self.keys[KEY_2] == 3):
            save = self.loadSave()
            self.core.states.append(StateMain(self.core, save))
        if (self.keys[KEY_ESC] == 3):
            sys.exit(0)
        self.drawScreen()


    def drawScreen(self):
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        tex = self.tex_hud
        tex.bind()

        glDisable(GL_DEPTH_TEST)
        glColor3f(1.0, 1.0, 1.0)
        glPushMatrix()
        glLoadIdentity()
        vertices = [] 
        texcoords = []

        vertices += [0.75, 0.5, -1.0, -0.75, 0.5, -1.0,
                -0.75, -0.5, -1.0, 0.75, -0.5, -1.0]
        texcoords += tex.select((10,0),(6,4))

        n = len(vertices)/3
        vertices = glarray(vertices)
        texcoords = glarray(texcoords)
        glVertexPointer(3,GL_FLOAT,0,vertices)
        glTexCoordPointer(2,GL_FLOAT,0,texcoords)
        glDrawArrays(GL_QUADS,0,n)
        glPopMatrix()
        glEnable(GL_DEPTH_TEST)

        pygame.display.flip()



    # Save data belonging to a game start.
    def startSave(self):
        return {
                'hubkeys': [], 
                'finmaps': set(), 
                'ammo': [0]*WEAPON_N, 
                'hard': 0
                }


    def loadSave(self):
        try:
            save = self.startSave()
            f = open(FN_SAVE)
            data = f.read()
            f.close()
            data = data.split()
            i = 0
            while i < len(data):
                if data[i] == 'hubkey':
                    save['hubkeys'].append((int(data[i+1]), int(data[i+2])))
                    i += 3
                elif data[i] == 'finmap':
                    save['finmaps'].add(data[i+1])
                    i += 2
                elif data[i] == 'ammo':
                    save['ammo'][int(data[i+1])] = int(data[i+2])
                    i += 3
                elif data[i] == 'hard':
                    save['hard'] = int(data[i+1])
                    i += 2
                else:
                    raise IOError
            return save
        except IOError, ValueError:
            print 'could not load save file, starting from beginning'
            return self.startSave()



