
import os

import pygame
from pygame.locals import *
from OpenGL.GL import *

from define import *


# Texture class, handles all OpenGL related texture stuff.
class Texture:

    # Offset so rounding errors don't cause rendering parts of the texture 
    # outside the range.
    OFFSET = 1.0/4096

    def __init__(self, filename):
        self.gltex = 0
        self.filename = filename
        filename = os.path.join(DATA_DIR, filename)
        print('loading ' + filename)
        tex = pygame.image.load(filename)
        data = pygame.image.tostring(tex,'RGBA',0)
        self.size = [tex.get_width(), tex.get_height()]
        self.gltex = glGenTextures(1) # opengl id
        glBindTexture(GL_TEXTURE_2D, self.gltex)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,GL_NEAREST)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_NEAREST)
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, self.size[0], self.size[1], 0, 
                GL_RGBA, GL_UNSIGNED_BYTE, data)
        self.sel = [0.0, 0.0, 1.0, 1.0]
    

    def __del__(self):
        if self.gltex != 0:
            glDeleteTextures(self.gltex)


    # Bind the current texture.
    def bind(self):
        glBindTexture(GL_TEXTURE_2D, self.gltex)


    # Set a texture selection, OpenGL texture coordinates, given a position on 
    # the texture.
    def select(self, pos, size=(1,1)):
        self.sel[0] = float(pos[0])*TEX_XX/self.size[0] + self.OFFSET
        self.sel[1] = float(pos[1])*TEX_XX/self.size[1] + self.OFFSET
        self.sel[2] = float(pos[0]+size[0])*TEX_XX/self.size[0] - self.OFFSET
        self.sel[3] = float(pos[1]+size[1])*TEX_XX/self.size[1] - self.OFFSET
        return self.getTexCoords()


    def getTexCoords(self):
        return [self.sel[2],self.sel[1], self.sel[0],self.sel[1],
                self.sel[0],self.sel[3], self.sel[2],self.sel[3]]
