
import random

from define import *
from Thing import Thing
from Sprite import Sprite
from Sfx import Sfx
from ThingHurt import ThingHurt
from ThingAni import ThingAni


class ThingClosed(Thing):


    SP_X = 1.0/4
    SP_Z = 1.0/16
    SFX = [ 
            'closed_move1.ogg', 
            'closed_move2.ogg',
            'closed_move3.ogg',
            'closed_move4.ogg'
    ]
    ATK_T = 128


    # args: [pos_x, pos_y, flag_id, chance]
    def __init__(self, main, args):
        if random.uniform(0.0, 1.0) > float(args[3]):
            # Don't spawn
            return
        Thing.__init__(self, main, args[:2])
        self.box[0] = 0.375
        self.pos[2] = -0.5
        self.flag = int(args[2])
        self.timer = 0
        self.state = 0
        self.nrg = 1
        self.ani = 0


    def damage(self, nrg):
        self.nrg -= nrg
        if self.nrg <= 0:
            ThingAni(self.main, self.pos, (9,4), 3, 8)
            Sfx(self.main, 'closed_death.ogg', self, 1.0)
            return
        if self.state == 3:
            self.setStateFire()


    def step(self):
        if self.nrg <= 0: return
        self.STATEFUNCTIONS[self.state](self)
        self.timer -= 1


    def setStatePopup(self):
        if not self.placeFree(self.pos, 2, set([self])):
            return
        self.state = 1
        self.sp[2] = self.SP_Z
        self.main.things_solid.add(self)
        self.main.things_destr.add(self)


    def setStateIdle(self): 
        self.state = 2
        if self.main.hardmode and random.randint(0,1) == 0:
            # In hard mode, some doors might be auto activated
            self.setStateChase()


    def setStateChase(self):
        self.state = 3
        self.sp[2] = 0
        self.pos[2] = 0.5


    def setStateFire(self):
        self.state = 4
        self.timer = self.ATK_T
        Sfx(self.main, 'closed_open.ogg', self, 3.0)


    def stepStateWait(self):
        if self.flag in self.main.map.flags:
            self.setStatePopup()


    def stepStateIdle(self):
        pass


    def stepStatePopup(self):
        self.move()
        if self.pos[2] >= 0.5: 
            self.sp[2] = 0
            self.pos[2] = 0.5
            self.setStateIdle()
        Sprite(self.main, self.pos, (6,4))


    def stepStateIdle(self):
        if random.randint(0,15) == 0 and self.canSeePc():
            self.setStateChase()
        Sprite(self.main, self.pos, (6,4))


    def stepStateChase(self):
        icv = self.inCamView()
        if self.main.hardmode and random.randint(0,15) == 0 and not icv:
            # Teleport
            prev = list(self.pos)
            self.pos[0] = random.uniform(0, self.main.map.size[0])
            self.pos[1] = random.uniform(0, self.main.map.size[1])
            if self.inCamView() or not self.placeFree(self.pos, 1, set([self])):
                # Cancel, moved into solid or camera view.
                self.pos = prev
        if random.randint(0,3) == 0 and not icv:
            self.target = list(self.main.pc.pos)
            self.target = [i + random.uniform(-2,2) for i in self.target]
            self.chaseTarget(self.SP_X)
            self.moveSolid()
            self.sp = [0.0, 0.0, 0.0]
            if random.randint(0,63) == 0:
                Sfx(self.main, random.choice(self.SFX), self)
            if self.isNearTarget(4.0) and self.canSeePc():
                self.setStateFire()
        Sprite(self.main, self.pos, (6,4))


    def stepStateFire(self):
        if self.timer >= self.ATK_T-16:
            ani = (self.ATK_T-self.timer)/8
        elif self.timer >= 4:
            ani = 2
        else:
            ani = self.timer/2 
        if self.timer <= 0:
            Sfx(self.main, 'closed_close.ogg', self)
            self.setStateChase()
            if self.canSeePc():
                ThingHurt(self.main, self.main.pc.pos, 1.0)
        Sprite(self.main, self.pos, (6+ani, 4))



    STATEFUNCTIONS = {
            0: stepStateWait,
            1: stepStatePopup,
            2: stepStateIdle,
            3: stepStateChase,
            4: stepStateFire}

