

import random

from Thing import Thing
from ThingAni import ThingAni
from Sprite import Sprite
from ThingHurt import ThingHurt
from Sfx import Sfx


class ThingFace(Thing):


    ACC_X = 1.0/256
    SP_X = 1.0/64
    SP_X2 = 1.0/16
    SP_Z = 1.0/16


    # args: [pos_x, pos_y, flag_id]
    # If the trigger id is negative, it will be ignored. Otherwise the enemy
    # will only pop up if the corresponding trigger has been hit.
    # Uses reg to toggle music volume if one of these things is active
    def __init__(self, main, args):
        Thing.__init__(self, main, args[:2])
        self.flag = int(args[2])
        self.box[0] = 0.25
        self.nrg = 1
        if self.main.hardmode: 
            self.nrg = 5
        self.state = 0
        if self.flag < 0: self.state = 1
        self.pos[2] = -0.5
        self.timer = 0


    # Remove self from reg list.    
    def cleanup(self):
        if self in self.main.things_reg:
            self.main.things_reg.remove(self)
            if len(self.main.things_reg) == 0:
                self.main.audio.setMusicVol(1)


    # Function called when damage is inflicted on the Thing. 
    def damage(self, nrg):
        if self.state < 3: return
        self.nrg -= nrg
        if self.nrg <= 0:
            Sfx(self.main, 'face_death.ogg', self)
            ThingAni(self.main, self.pos, (9,7), 3, 16)
            self.cleanup()


    def step(self):
        if self.nrg <= 0: 
            self.cleanup()
            return
        self.STATEFUNCTIONS[self.state](self)
        self.timer -= 1


    def setStatePopup(self):
        self.state = 2
        self.sp[2] = self.SP_Z
        self.main.things_destr.add(self)
        self.main.things_reg.add(self)
        self.main.audio.setMusicVol(0)


    def setStateChase(self):
        self.state = 3
        self.pos[2] = 0.5
        self.sp[2] = 0.0
        self.timer = random.randint(120,150)


    def setStateRetreat(self, time=60):
        self.state = 4
        self.setRandomTarget()
        self.timer = time


    def moveToTarget(self):
        self.chaseTarget(self.ACC_X)
        if self.placeFree(self.pos, 3):
            self.limitSpeed(self.SP_X2)
        else:
            self.limitSpeed(self.SP_X)
        self.posprev = list(self.pos)
        self.move()


    def draw(self):
        pos = [self.pos[i] + random.uniform(-0.0625, 0.0625) for i in range(3)]
        Sprite(self.main, pos, (8,7))
    


    def stepStateFlag(self):
        # Waiting for flag
        if self.flag in self.main.map.flags:
            self.setStatePopup()
            Sfx(self.main, 'face_activate.ogg', self)


    def stepStateWait(self):
        # Waiting for PC to approach.
        self.setPcTarget()
        if self.isNearTarget(2.0):
            self.setStatePopup()
            Sfx(self.main, 'face_activate.ogg', self)


    def stepStatePopup(self):
        # Popping up from the ground
        self.move()
        if self.pos[2] >= 0.5: 
            self.setStateChase()
        self.draw()


    def stepStateChase(self):
        # Chasing the PC
        self.setPcTarget()
        self.moveToTarget()
        if random.randint(0,255) == 0:
            Sfx(self.main, 'face_move.ogg', self)
        if self.timer <= 0:
            # Timeout, retreat a bit
            self.setStateRetreat(random.randint(90, 150))
        if self.posprev == self.pos and self.sp == [0.0, 0.0, 0.0]:
            # Stuck, retreat a bit
            self.setStateRetreat(random.randint(90, 150))
        if self.collide(self.main.pc):
            # Near PC, attack
            Sfx(self.main, 'face_attack.ogg', self)
            ThingHurt(self.main, self.pos, 1.0)
            self.cleanup()
            self.nrg = 0
        self.draw()


    def stepStateRetreat(self):
        # Retreating
        self.moveToTarget()
        if self.posprev == self.pos and self.sp == [0.0, 0.0, 0.0]:
            self.setRandomTarget()
        if self.timer <= 0:
            self.setStateChase()
        self.draw()



    STATEFUNCTIONS = {
            0: stepStateFlag,
            1: stepStateWait,
            2: stepStatePopup,
            3: stepStateChase,
            4: stepStateRetreat}



