
import random

from Thing import Thing
from ThingAni import ThingAni
from Sprite import Sprite
from ThingFlowerProj import ThingFlowerProj
from Sfx import Sfx

class ThingFlower(Thing):

    STUN_T = 1024

    # Args: [pos_x, pos_y]
    def __init__(self, main, args):
        Thing.__init__(self, main, args[:2])
        self.box[0] = 0.375
        self.timer = 0
        self.ani_sp = 0
        self.ani = 0
        self.state = 0
        if self.main.hardmode:
            self.STUN_T /= 4
        self.time_offset = random.randint(0,15)
        self.main.things_solid.add(self)
        self.main.things_destr.add(self)


    def damage(self, nrg):
        self.state = 3
        self.ani_sp = 0
        self.timer = self.STUN_T


    def drawSpriteActive(self): 
        if self.ani_sp < 56:
            Sprite(self.main, self.pos, (self.ani/128,5))
        else:
            Sprite(self.main, self.pos, (2,5))


    def step(self):
        if self.nrg <= 0: return
        self.STATEFUNCTIONS[self.state](self)
        self.ani += self.ani_sp
        self.ani %= 256
        self.timer -= 1


    def stepStateWait(self):
        if (self.main.step_i+self.time_offset) % 16 == 0 and self.canSeePc():
            self.state = 1
            self.timer = 64
            Sfx(self.main, 'flower_charge.ogg', self)
        self.drawSpriteActive()


    def stepStateCharge(self):
        self.ani_sp = 64-self.timer
        if self.timer <= 0:
            self.setPcTarget()
            if self.main.hardmode:
                repeat = 5
            else: 
                repeat = 1
            for i in range(repeat):
                ThingFlowerProj(self.main, self.pos, self.target, self)
            self.state = 2
            self.timer = random.randint(128,192)
        self.drawSpriteActive()


    def stepStateRecover(self):
        self.ani_sp = max(0, self.timer-64)
        if self.timer <= 0:
            self.state = 0
        self.drawSpriteActive()


    def stepStateStunned(self):
        if self.timer <= 0:
            self.state = 0
        Sprite(self.main, self.pos, (4,5))


    STATEFUNCTIONS = {
            0: stepStateWait,
            1: stepStateCharge,
            2: stepStateRecover,
            3: stepStateStunned}

