

import random

from Thing import Thing
from ThingAni import ThingAni
from Sprite import Sprite
from ThingHurt import ThingHurt
from Sfx import Sfx


class ThingGhost(Thing):


    ACC_X = 1.0/256
    SP_X = 1.0/32
    SP_Z = 1.0/16


    # args: [pos_x, pos_y, flag_id]
    # If the trigger id is negative, it will be ignored. Otherwise the enemy
    # will only pop up if the corresponding trigger has been hit.
    def __init__(self, main, args):
        Thing.__init__(self, main, args[:2])
        self.flag = int(args[2])
        self.box[0] = 0.375
        self.nrg = 8
        self.state = 0
        if self.main.hardmode:
            self.ACC_X *= 4
            self.SP_X *= 4
        if self.flag < 0: self.state = 1
        self.pos[2] = -0.5
        self.timer = 0


    # Function called when damage is inflicted on the Thing. 
    def damage(self, nrg):
        if self.state < 3: return
        self.nrg -= nrg
        if self.nrg <= 0:
            Sfx(self.main, 'ghost_death.ogg', self)
            ThingAni(self.main, self.pos, (3,3), 3, 16)
        else:
            Sfx(self.main, 'ghost_hurt.ogg', self)
            self.state = 6
            self.setRandomTarget()
            self.timer = 30


    def step(self):
        if self.nrg <= 0: return
        self.STATEFUNCTIONS[self.state](self)
        if self.main.hardmode:
            self.timer -= 3
        else:
            self.timer -= 1


    def setStatePopup(self):
        if not self.placeFree(self.pos, 2, set([self])):
            return
        self.state = 2
        self.sp[2] = self.SP_Z
        self.main.things_solid.add(self)
        self.main.things_destr.add(self)


    def setStateChase(self):
        self.state = 3
        self.pos[2] = 0.5
        self.sp[2] = 0.0
        self.timer = random.randint(300,450)


    def setStateRetreat(self, time=150):
        self.state = 4
        self.setRandomTarget()
        self.timer = time


    def setStateAttack(self, time=15):
        self.state = 5
        self.timer = time


    def moveToTarget(self):
        self.chaseTarget(self.ACC_X)
        self.limitSpeed(self.SP_X)
        self.posprev = list(self.pos)
        self.moveSolid(2)


    def stepStateFlag(self):
        # Waiting for flag
        if self.flag in self.main.map.flags:
            self.setStatePopup()
            Sfx(self.main, 'ghost_popup.ogg', self)


    def stepStateWait(self):
        # Waiting for PC to approach.
        self.setPcTarget()
        if self.isNearTarget(2.0):
            self.setStatePopup()
            Sfx(self.main, 'ghost_popup.ogg', self)


    def stepStatePopup(self):
        # Popping up from the ground
        self.move()
        if self.pos[2] >= 0.5: 
            self.setStateChase()
        Sprite(self.main, self.pos, (0,3))


    def stepStateChase(self):
        # Chasing the PC
        self.setPcTarget()
        self.moveToTarget()
        if self.timer <= 0:
            # Timeout, retreat a bit
            self.setStateRetreat(random.randint(90, 150))
        if self.posprev == self.pos and self.sp == [0.0, 0.0, 0.0]:
            # Stuck, retreat a bit
            self.setStateRetreat(random.randint(90, 150))
        if self.isNearTarget(1.0):
            # Near PC, attack
            self.setStateAttack()
        Sprite(self.main, self.pos, (0,3))


    def stepStateRetreat(self):
        # Retreating
        self.moveToTarget()
        if self.posprev == self.pos and self.sp == [0.0, 0.0, 0.0]:
            self.setRandomTarget()
        if self.timer <= 0:
            self.setStateChase()
        Sprite(self.main, self.pos, (0,3))


    def stepStateAttack(self):
        # Attacking (biting)
        self.setPcTarget()
        self.moveToTarget()
        if self.timer <= 0:
            ThingHurt(self.main, self.pos, 0.5)
            self.setStateRetreat(random.randint(90, 150))
            Sfx(self.main, 'ghost_attack.ogg', self)
        Sprite(self.main, self.pos, (2,3))


    def stepStateHurt(self):
        # Hurt
        self.moveToTarget()
        if self.posprev == self.pos and self.sp == [0.0, 0.0, 0.0]:
            self.setRandomTarget()
        if self.timer <= 0:
            self.setStateRetreat(random.randint(90, 120))
        Sprite(self.main, self.pos, (1,3))


    STATEFUNCTIONS = {
            0: stepStateFlag,
            1: stepStateWait,
            2: stepStatePopup,
            3: stepStateChase,
            4: stepStateRetreat,
            5: stepStateAttack,
            6: stepStateHurt}



