

import math
import random

from Thing import Thing
from Sprite import Sprite
from ThingImpact import ThingImpact
from ThingHurt import ThingHurt
from ThingAni import ThingAni
from Sfx import Sfx


class ThingNova(Thing):


    SP = 1.0/16
    ACC = 1.0/1024
    FRIC = 0.0


    def __init__(self, main, pos, target, parent):
        Thing.__init__(self, main, pos)
        self.target = target
        self.parent = parent
        self.nrg = 1
        self.box[0] = 0.5
        self.pos[2] = 1.5
        self.state = 0
        self.timer = 128
        Sfx(self.main, 'nova_charge.ogg', self, 4.0)


    def kaboom(self):
        ThingHurt(self.main, self.pos, 1.0)
        ThingHurt(self.main, self.pos, 2.0)
        for i in range(5):
            x = self.pos[0] + random.uniform(-1,1)
            y = self.pos[1] + random.uniform(-1,1)
            ThingAni(self.main, [x,y,0.5], (10,3), 3, 16-1-i)
        self.main.flash = [1.0, 1.0, 0.8, 0.5]


    def step(self):
        if self.nrg == 0: return
        if self.main.hardmode:
            self.timer -= 2
        else:    
            self.timer -= 1
        if self.state == 0:
            self.pos[0] = self.parent.pos[0]
            self.pos[1] = self.parent.pos[1]
            if self.timer <= 64:
                Sprite(self.main, self.pos, (14+(self.timer/4)%2,3))
            else:
                Sprite(self.main, self.pos, (14+(self.timer/4)%2,2))
            if self.timer <= 0:
                self.dir[0] = self.getTargetDir()
                self.sp[0] = self.SP * math.cos(float(self.dir[0]))
                self.sp[1] = self.SP * math.sin(float(self.dir[0]))
                self.sp[2] = -1.0/(math.sqrt(
                        sum([i**2 for i in self.target[:2]]))*8.0)
                self.state = 1
                self.timer = 1024
                Sfx(self.main, 'nova_launch.ogg', self, 4.0)
        elif self.state == 1:
            if self.main.hardmode:
                # Update movement to target in hard mode only
                self.dir[0] = self.getTargetDir()
                self.sp[0] += self.ACC * math.cos(float(self.dir[0]))
                self.sp[1] += self.ACC * math.sin(float(self.dir[0]))
                self.sp[2] = -1.0/(math.sqrt(
                        sum([i**2 for i in self.target[:2]]))*8.0)
                self.limitDirSpeed()

            if (self.moveSolid(3, set([self.parent])) or self.pos[2] < 0.5 
                    or self.timer <= 0):
                self.kaboom()
                self.timer = 32
                self.state = 2
                Sfx(self.main, 'nova_explode.ogg', self, 4.0)
            else: 
                Sprite(self.main, self.pos, (14+(self.timer/4)%2,1))
        else:
            if random.randint(0,3):
                x = self.pos[0] + random.uniform(-2,2)
                y = self.pos[1] + random.uniform(-2,2)
                ThingAni(self.main, [x,y,0.5], (10,3), 3, 16)
            if random.randint(0,7):
                self.main.flash = [1.0, 1.0, 0.8, 0.25]
            if self.timer <= 0:
                self.nrg = 0






