
import math

from define import *
from Thing import Thing
from WeaponRake import WeaponRake
from WeaponLaser import WeaponLaser
from WeaponBounce import WeaponBounce
from WeaponSpread import WeaponSpread
from WeaponTv import WeaponTv
from Sfx import Sfx



class ThingPc(Thing):

    ACC = 1.0/128
    FRIC = 1.0/256
    SP = 1.0/16
    ROT_X = math.pi/768
    ROT_Y = math.pi/768
    SWITCH_T = 64


    def __init__(self, main, pos, dir):
        Thing.__init__(self, main, pos, dir)
        self.main.things_solid.add(self)
        self.pos[2] = 0.5
        self.box[0] = 0.125
        self.sp = [0.0, 0.0, 0.0]
        self.main.flash = [0.25, 0.25, 1.0, 1.0]
        self.weapons = [None]*WEAPON_N
        self.weapons[WEAPON_RAKE] = WeaponRake(self.main)
        self.weapons[WEAPON_LASER] = WeaponLaser(self.main)
        self.weapons[WEAPON_SPREAD] = WeaponSpread(self.main)
        self.weapons[WEAPON_BOUNCE] = WeaponBounce(self.main)
        self.weapons[WEAPON_TV] = WeaponTv(self.main)
        self.weapon_switch = 0
        self.weapon_prev = self.main.weapon_i
        self.nrg = 5


    def hurt(self, nrg):
        self.main.flash = [1.0, 0.0, 0.0, 0.5]
        self.nrg -= 1


    def updateDir(self):
        self.mouse_sens = self.main.core.config.mouse_sens
        self.dir[0] -= self.main.mouse[0]*self.mouse_sens*self.ROT_X
        if self.main.cheat:
            self.dir[1] -= self.main.mouse[1]*self.mouse_sens*self.ROT_Y
        else:
            self.dir[1] = 0.0

    
    def updateSpeed(self):
        if (self.main.keys[KEY_LEFT]%2):
            self.sp[0] += self.ACC * math.cos(float(self.dir[0]+math.pi/2))
            self.sp[1] += self.ACC * math.sin(float(self.dir[0]+math.pi/2))
        if (self.main.keys[KEY_RIGHT]%2):
            self.sp[0] += self.ACC * math.cos(float(self.dir[0]-math.pi/2))
            self.sp[1] += self.ACC * math.sin(float(self.dir[0]-math.pi/2))
        if (self.main.keys[KEY_UP]%2):
            self.sp[0] += self.ACC * math.cos(float(self.dir[0]))
            self.sp[1] += self.ACC * math.sin(float(self.dir[0]))
        if (self.main.keys[KEY_DOWN]%2):
            self.sp[0] -= self.ACC * math.cos(float(self.dir[0]))
            self.sp[1] -= self.ACC * math.sin(float(self.dir[0]))
        if self.main.cheat: 
            self.SP = 1.0/4
        else:
            self.SP = 1.0/16



    def handleWeapon(self):
        self.main.ammo[0] = 1
        if self.weapon_switch > 0: 
            self.weapon_switch -= 1
            return
        self.weapon_prev = self.main.weapon_i
        switch = False
        for i in range(KEY_1, KEY_5+1):
            if self.main.keys[i] == 3:
                prev = self.main.weapon_i
                self.main.weapon_i = i-KEY_1
                if self.main.ammo[self.weapons[self.main.weapon_i].tag] <= 0:
                    self.main.weapon_i = prev
                else: 
                    switch = True
                    self.weapon_switch = self.SWITCH_T
                    self.weapon_prev = prev
                    break
        while self.main.ammo[self.weapons[self.main.weapon_i].tag] <= 0:
            # Skip empty weapons, note that this will assume there is at 
            # least one weapon with ammo.
            switch = True
            self.main.weapon_i += 1
            self.main.weapon_i %= len(self.weapons)
            self.weapon_switch = self.SWITCH_T
        if switch == False and self.main.keys[KEY_FIRE]%2:
            self.weapons[self.main.weapon_i].attack()



    def step(self):
        self.updateDir()
        self.updateSpeed()
        self.limitDirSpeed()
        self.handleWeapon()
        if self.main.cheat:
            self.move()
            if self.main.keys[KEY_ENTER] % 2:
                self.pos[2] += 0.125
        else:
            self.pos[2] = 0.5
            self.moveSolid()
        self.main.cam = (self.pos[0], self.pos[1], self.pos[2], 
                self.dir[0]/math.pi*180, self.dir[1]/math.pi*180)



