
import random

from define import *

from Thing import Thing
from Sprite import Sprite
from Sfx import Sfx
from ThingHurt import ThingHurt
from ThingAni import ThingAni



class ThingShadow(Thing):


    SP_X = 1.0/4
    MAX_ACTIVE = 2


    # args: [pos_x, pos_y]
    # Uses reg to see how many active shadows are currently in play.
    def __init__(self, main, args):
        Thing.__init__(self, main, args[:2])
        self.state = 0
        self.timer = 0
        self.nrg = 1
        self.box[0] = 0.25
        if self.main.hardmode:
            self.MAX_ACTIVE = 4


    def damage(self, nrg):
        self.nrg -= nrg
        if self.nrg <= 0:
            ThingAni(self.main, self.pos, (8,5), 3, 8)
            Sfx(self.main, 'shadow_death.ogg', self)


    def step(self):
        if self.nrg <= 0: return
        self.STATEFUNCTIONS[self.state](self)
        self.timer -= 1


    def setStateChase(self):
        if not self.placeFree(self.pos, 2, set([self])):
            return
        self.state = 1
        self.main.things_solid.add(self)
        self.main.things_destr.add(self)
        self.main.things_reg.add(self)


    def setStateAttack(self):
        self.state = 2
        self.timer = 32


    def stepStateInactive(self):
        if (len(self.main.things_reg) < self.MAX_ACTIVE and 
                random.randint(0,15) == 0):
            self.setStateChase()
        Sprite(self.main, self.pos, (6,5))


    def stepStateChase(self):
        if random.randint(0,4) == 0 and not self.inCamView():
            self.target = list(self.main.pc.pos)
            self.target = [i + random.uniform(-2,2) for i in self.target]
            self.chaseTarget(self.SP_X)
            self.moveSolid()
            self.sp = [0.0, 0.0, 0.0]
            if random.randint(0,63) == 0:
                Sfx(self.main, 'shadow_move.ogg', self)
            if self.isNearTarget(0.5) and self.canSeePc():
                self.setStateAttack()
        Sprite(self.main, self.pos, (6,5))


    def stepStateAttack(self):
        if self.timer <= 0:
            Sfx(self.main, 'shadow_attack.ogg', self)
            ThingHurt(self.main, self.pos, 2.0)
            self.setStateChase()
            self.nrg = 0
        Sprite(self.main, self.pos, (7,5))



    STATEFUNCTIONS = {
            0: stepStateInactive,
            1: stepStateChase,
            2: stepStateAttack}
