

import random

from Thing import Thing
from Sprite import Sprite
from ThingAni import ThingAni
from ThingHurt import ThingHurt
from Sfx import Sfx


class ThingSkeleton(Thing):


    ACC_X = 1.0/256
    SP_X = 1.0/64
    ATK_T = 30


    # args [pos_x, pos_y, speed]
    def __init__(self, main, args):
        Thing.__init__(self, main, args[:2])
        self.spmul = float(args[2])
        self.ACC_X *= self.spmul
        self.SP_X *= self.spmul
        if self.main.hardmode:
            self.SP_X *= 4
            self.ATK_T /= 3
        self.box[0] = 0.25
        self.nrg = 4
        self.pos[2] = 0.5
        self.main.things_solid.add(self)
        self.main.things_destr.add(self)
        self.ani_offset = random.randint(0,31)
        self.state = 1
        self.timer = 0


    # Function called when damage is inflicted on the Thing. 
    def damage(self, nrg):
        self.nrg -= nrg
        if self.nrg <= 0:
            ThingAni(self.main, self.pos, (3,4), 3, 8)
            Sfx(self.main, 'skeleton_death.ogg', self)
        else:
            Sfx(self.main, 'skeleton_hurt.ogg', self)
        self.setStateRetreat(random.randint(180,300))


    def getWalkSprite(self):
        if (self.main.step_i+self.ani_offset)*self.spmul % 32 < 16:
            return (0,4)
        else:
            return (1,4)


    def step(self):
        if self.nrg <= 0: return
        self.STATEFUNCTIONS[self.state](self)
        self.timer -= 1



    def setStateChase(self):
        self.state = 3
        self.pos[2] = 0.5
        self.sp[2] = 0.0
        self.timer = 300


    def setStateRetreat(self, time=150):
        self.state = 4
        self.setRandomTarget()
        self.timer = time


    def setStateAttack(self):
        self.state = 5
        self.timer = self.ATK_T


    def moveToTarget(self):
        self.chaseTarget(self.ACC_X)
        self.limitSpeed(self.SP_X)
        self.posprev = list(self.pos)
        self.moveSolid(1)


    def stepStateWait(self):
        if random.randint(0,15) == 0 and self.canSeePc():
            Sfx(self.main, 'skeleton_activate.ogg', self)
            self.setStateChase()
        Sprite(self.main, self.pos, (0,4))


    def stepStateChase(self):
        # Chasing the PC
        self.setPcTarget()
        self.moveToTarget()
        if self.timer <= 0:
            # Timeout, retreat a bit
            self.setStateRetreat(random.randint(90,150))
        if self.posprev == self.pos and self.sp == [0.0, 0.0, 0.0]:
            # Stuck, retreat a bit
            self.setStateRetreat(random.randint(90,150))
        if self.isNearTarget(1.0):
            # Near PC, attack
            self.setStateAttack()
        Sprite(self.main, self.pos, self.getWalkSprite())


    def stepStateRetreat(self):
        # Retreating
        self.moveToTarget()
        if self.posprev == self.pos and self.sp == [0.0, 0.0, 0.0]:
            self.setRandomTarget()
        if self.timer <= 0:
            self.setStateChase()
        Sprite(self.main, self.pos, self.getWalkSprite())


    def stepStateAttack(self):
        # Attacking (biting)
        self.setPcTarget()
        self.moveToTarget()
        if self.timer <= 0:
            Sfx(self.main, 'skeleton_attack.ogg', self)
            ThingHurt(self.main, self.pos, 0.5)
            self.setStateRetreat(random.randint(90,150))
        Sprite(self.main, self.pos, (2,4))


    def stepStateHurt(self):
        # Hurt
        self.moveToTarget()
        if self.posprev == self.pos and self.sp == [0.0, 0.0, 0.0]:
            self.setRandomTarget()
        if self.timer <= 0:
            self.setStateRetreat(random.randint(90,150))
        Sprite(self.main, self.pos, (2,4))


    STATEFUNCTIONS = {
            1: stepStateWait,
            3: stepStateChase,
            4: stepStateRetreat,
            5: stepStateAttack,
            6: stepStateHurt}



