


class Tile:
 
    # Type-to-tileheight conversion.
    TYPE2HEIGHT = {'w': 1, 'f': 0, 'l': -0.25, 'h': 2, 'p': -1}

    def __init__(self, main, solid, type, spr):
        self.main = main

        # Solidity of tile: 
        # 0: not solid
        # 1: not very solid, will block PC and ground-based Things (e.g. lake)
        # 2: fairly solid, will block PC and enemies, generally not projectiles
        # or view lines.
        # 3: fully solid, will block everything
        self.solid = solid

        # Tile type
        # 'w': Wall tile, 1 unit high
        # 'f': Floor tile, 0 height
        # 'l': Lowered (lake) tile, 0.25 units deep
        # 'h': High wall tile, 2 units high
        # 'p': Pit, 1 unit deep.
        self.type = type

        # Tile height in units:
        self.height = self.TYPE2HEIGHT[type]

        # Tile sprite, part of texture to use.
        self.spr = spr


    def draw(self, map, pos, vertices, texcoords, colors):
        tex = self.main.tex_landscape
        sh = self.height
        # Left
        oh = map.getHeight((pos[0]-1, pos[1]))
        if oh <= sh:
            vertices += [pos[0],pos[1],oh,pos[0],pos[1],sh,
                    pos[0],pos[1]+1,sh,pos[0],pos[1]+1,oh]
            tex.select((self.spr[0], self.spr[1]+1-sh), (1,sh-oh))
            texcoords += [tex.sel[2],tex.sel[3],tex.sel[2],tex.sel[1],
                    tex.sel[0],tex.sel[1],tex.sel[0],tex.sel[3]]
            colors += [0.85] * 12
        # Right
        oh = map.getHeight((pos[0]+1, pos[1]))
        if oh <= sh:
            vertices += [pos[0]+1,pos[1],oh,pos[0]+1,pos[1]+1,oh,
                    pos[0]+1,pos[1]+1,sh,pos[0]+1,pos[1],sh]
            tex.select((self.spr[0], self.spr[1]+1-sh), (1,sh-oh))
            texcoords += [tex.sel[0],tex.sel[3],tex.sel[2],tex.sel[3],
                    tex.sel[2],tex.sel[1],tex.sel[0],tex.sel[1]]
            colors += [0.9] * 12
        # Back
        oh = map.getHeight((pos[0], pos[1]-1))
        if oh <= sh:
            vertices += [pos[0],pos[1],oh,pos[0]+1,pos[1],oh,
                    pos[0]+1,pos[1],sh,pos[0],pos[1],sh]
            tex.select((self.spr[0], self.spr[1]+1-sh), (1,sh-oh))
            texcoords += [tex.sel[0],tex.sel[3],tex.sel[2],tex.sel[3],
                    tex.sel[2],tex.sel[1],tex.sel[0],tex.sel[1]]
            colors += [0.95] * 12
        # Front
        oh = map.getHeight((pos[0], pos[1]+1))
        if oh <= sh:
            vertices += [pos[0],pos[1]+1,oh,pos[0],pos[1]+1,sh,
                    pos[0]+1,pos[1]+1,sh,pos[0]+1,pos[1]+1,oh]
            tex.select((self.spr[0], self.spr[1]+1-sh), (1,sh-oh))
            texcoords += [tex.sel[2],tex.sel[3],tex.sel[2],tex.sel[1],
                    tex.sel[0],tex.sel[1],tex.sel[0],tex.sel[3]]
            colors += [0.8] * 12
        # Floor
        vertices += [pos[0],pos[1],sh,pos[0]+1,pos[1],sh,
                pos[0]+1,pos[1]+1,sh,pos[0],pos[1]+1,sh]
        tex.select(self.spr, (1,1))
        texcoords += [tex.sel[0],tex.sel[3],tex.sel[2],tex.sel[3],
                tex.sel[2],tex.sel[1],tex.sel[0],tex.sel[1]]
        if sh <= 0:
            colors += [1.0] * 12
        else:
            colors += [0.0] * 12



